//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by ntgetpin.rc
//
#define IDD_DATABASE_PASSWORD           101
#define IDI_KEY                         103
#define IDI_LOGO                        105
#define IDR_MENU                        205
#define IDEDIT                          1000
#define IDC_TOKEN_NAME                  1001
#define ID_SERVER_RESTART               40001
#define ID_FILE_EXIT                    40003
#define ID_SERVER_SHUTDOWN              40004
#define ID_SERVER_SUSPEND               40005
#define ID_SERVER_RESUME                40006

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        106
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1006
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
